import React, { Component } from 'react';
import pathToRegexp from 'path-to-regexp';
import moment from 'moment';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import { Card, Modal, Button, message } from 'antd';
import { StandardTableList } from '../../components/AXList';
import Ellipsis from '../../components/Ellipsis';
import { Hotax } from '../../utils/config';
import { renderStatus, addRowKey } from '../../utils/utils';

const Message = message;

const productNameMap = {
  Course: '课程',
  Support: '配套',
  Training: '师训',
  Package: '套餐包',
};

@connect(({ loading, shelves }) => ({
  shelves,
  loading: loading.models.shelves,
}))
export default class ShelvesListPage extends Component {
  constructor(props) {
    super(props);
    const { state } = props.location;
    const match = pathToRegexp('/shelves/:type/list').exec(location.pathname);
    const type = match[1];
    this.state = {
      scene: type && (type.substring(0, 1).toUpperCase() + type.substring(1)),
      UIParams: (state || {}).UIParams, // 组件的状态参数
      Queryers: (state || {}).Queryers, // 查询的条件参数
    };
  }
  componentWillMount() {
    this.props.dispatch({
      type: 'shelves/cleanListState',
    });
  }
  componentDidMount() {
    const { scene } = this.state;
    this.props.dispatch({
      type: `shelves/fetch${scene}ItemList`,
      payload: { ...this.state.Queryers },
    });
  }
  /**
   * 跳转到创建页面
   */
  handleCreateOperation = () => {
    const { scene } = this.state;
    this.props.dispatch(routerRedux.push({
      pathname: `/shelves/${scene.toLowerCase()}/create`,
      state: this.state,
    }));
  };
  /**
   * 下架某渠道下的一个产品
   * @param {Object} item
   */
  handleDeleteOperation = (item) => {
    Modal.confirm({
      okText: '确定',
      cancelText: '取消',
      title: '你确定要下架该产品吗?',
      content: '下架后该产品将不再任何栏目下展现',
      onOk: () => {
        this.props.dispatch({
          type: 'shelves/deleteItem',
          payload: { pid: item.pid, merchantId: item.merchantId },
          states: this.state,
        });
      },
    });
  };
  /**
   * 恢复某渠道的一个产品
   * @param item
   */
  handleRecoverOperation = (item) => {
    Modal.confirm({
      okText: '确定',
      cancelText: '取消',
      title: '你确定要上架该产品吗?',
      onOk: () => {
        this.props.dispatch({
          type: 'shelves/recoverItem',
          payload: {
            pid: item.pid,
            merchantId: item.merchantId,
            status: Hotax.STATUS_NORMAL,
          },
          states: this.state,
        });
      },
    });
  };
  /**
   * 跳转到编辑界面
   * @param {Object} item
   */
  handleEditOperation = (item) => {
    const { scene } = this.state;
    this.props.dispatch(routerRedux.push({
      pathname: `/shelves/${scene.toLowerCase()}/edit`,
      state: {
        pid: item.pid,
        merchantId: item.merchantId,
        ...this.state,
      },
    }));
  };
  /**
   * 执行过滤操作
   * @param {Object} params
   * @param {Object} states
   */
  handleFilterOperation = (params, states) => {
    const { scene } = this.state;
    this.props.dispatch({
      type: `shelves/fetch${scene}ItemList`,
      payload: params,
    });
    this.setState({
      UIParams: states,
      Queryers: params,
    });
  };
  /**
   * TODO: 批量操作
   */
  handleBatchOperation = () => {
    Message.info('暂不支持批量操作!');
  };

  render() {
    const { scene } = this.state;
    const { loading, shelves } = this.props;
    const { list, totalSize, pageSize, pageNo } = shelves;

    const renderOperation = (item) => {
      return (
        <div>
          <Button
            size="small"
            className="editBtn"
            onClick={() => this.handleEditOperation(item)}
          >编辑
          </Button>
          {item.status === Hotax.STATUS_NORMAL ?
            (
              <Button
                size="small"
                className="delBtn"
                onClick={() => this.handleDeleteOperation(item)}
              >下架
              </Button>
            ) : (
              <Button
                size="small"
                className="recBtn"
                onClick={() => this.handleRecoverOperation(item)}
              >上架
              </Button>
            )
          }
        </div>
      );
    };

    const batchActions = [{
      key: 'delete',
      name: '批量删除',
    }, {
      key: 'recovery',
      name: '批量恢复',
    }];
    const basicSearch = {
      keys: [{
        name: `${productNameMap[scene]}编号`,
        field: 'code',
      }, {
        name: `${productNameMap[scene]}名称`,
        field: 'name',
      }],
    };
    const pagination = {
      pageNo,
      pageSize,
      totalSize,
    };
    const columns = [{
      title: `${productNameMap[scene]}编号`,
      key: 1,
      dataIndex: 'code',
      width: '15%',
    }, {
      title: `${productNameMap[scene]}名称`,
      key: 2,
      dataIndex: 'name',
      render: text => (
        <Ellipsis tooltip lines={1}>{text}</Ellipsis>
      ),
      width: '30%',
    }, {
      title: '上架渠道',
      key: 3,
      dataIndex: 'merchantName',
      width: '14%',
    }, {
      title: '上架状态',
      key: 4,
      dataIndex: 'status',
      render: text => renderStatus(text, '已下架', '已上架'),
      width: '10%',
    }, {
      title: '更新时间',
      key: 5,
      dataIndex: 'gmtModified',
      render: text => moment(text).format('YYYY-MM-DD HH:mm:ss'),
      width: '17%',
    }, {
      title: '操作',
      key: 6,
      dataIndex: 'operation',
      render: (_, record) => renderOperation(record),
      width: '13%',
      align: 'right',
    }];
    return (
      <Card>
        <StandardTableList
          columns={columns}
          loading={loading}
          dataSource={addRowKey(list)}
          header={{
            basicSearch,
            onFilterClick: this.handleFilterOperation,
            onCreateClick: this.handleCreateOperation,
          }}
          footer={{
            pagination,
            batchActions,
            onBatchClick: this.handleBatchOperation,
          }}
          keepUIState={{ ...this.state.UIParams }}
        />
      </Card>
    );
  }
}
